"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var csharp_binary_stream_1 = require("csharp-binary-stream");
var fs = require('fs');
var parseString = require('xml2js').parseString;
var FontKerning = /** @class */ (function () {
    function FontKerning(inStream) {
        this.char1 = inStream.readInt();
        this.char2 = inStream.readInt();
        this.u1 = inStream.readByte();
        this.u2 = inStream.readByte();
        this.u3 = inStream.readByte();
        this.u4 = inStream.readByte();
    }
    FontKerning.prototype.write = function (bw) {
        bw.writeInt(this.char1);
        bw.writeInt(this.char2);
        bw.writeByte(this.u1);
        bw.writeByte(this.u2);
        bw.writeByte(this.u3);
        bw.writeByte(this.u4);
    };
    return FontKerning;
}());
exports.FontKerning = FontKerning;
var FontCharacter = /** @class */ (function () {
    function FontCharacter() {
    }
    FontCharacter.prototype.ReadInt = function (br, large) {
        return large ? br.readShort() : br.readSignedByte();
    };
    FontCharacter.prototype.WriteInt = function (bw, i, large) {
        if (large)
            bw.writeShort(i);
        else
            bw.writeSignedByte(i);
    };
    FontCharacter.prototype.read = function (br, large) {
        if (large === void 0) { large = false; }
        this.ReadInt(br, large); //Unknown
        this.W = br.readByte();
        this.H = br.readByte();
        this.XAdvance = br.readByte();
        this.XOffset = br.readSignedByte();
        this.YOffset = this.ReadInt(br, large); //Oddly the only one that turned into a short?
        this.X = br.readShort();
        this.Y = br.readShort();
    };
    FontCharacter.prototype.WriteMainSection = function (bw, large) {
        if (large === void 0) { large = false; }
        this.WriteInt(bw, 0, large);
        bw.writeByte(this.W);
        bw.writeByte(this.H);
        bw.writeByte(this.XAdvance);
        bw.writeSignedByte(this.XOffset);
        this.WriteInt(bw, this.YOffset, large);
        bw.writeShort(this.X);
        bw.writeShort(this.Y);
    };
    FontCharacter.prototype.WriteCodeIDSection = function (bw) {
        bw.writeChar(String.fromCharCode(this.ASCIICode), csharp_binary_stream_1.Encoding.Utf8);
        bw.writeInt(this.ID);
    };
    return FontCharacter;
}());
exports.FontCharacter = FontCharacter;
var endTag = 925913978;
var DieselFont = /** @class */ (function () {
    function DieselFont() {
    }
    DieselFont.prototype.hasKerning = function () {
        return this.kerning.length > 0;
    };
    DieselFont.prototype.read = function (filepath, large) {
        fs.readFile('file.txt', function (err, data) {
            if (err) {
                return console.error(err);
            }
            var buf = new ArrayBuffer(data.length * 2); // 2 bytes for each char
            var bufView = new Uint16Array(buf);
            for (var i = 0, strLen = data.length; i < strLen; i++) {
                bufView[i] = data.charCodeAt(i);
            }
            this.ReadDieselFile(new csharp_binary_stream_1.BinaryReader(bufView), large);
            console.log("Asynchronous read: " + data.toString());
        });
        fs.readFileSync('foo.txt', 'utf8');
    };
    DieselFont.prototype.ReadBMFontXmlFile = function (filepath) {
        fs.readFile('file.txt', function (err, data) {
            if (err) {
                return console.error(err);
            }
            parseString(data, function (err, result) {
                console.log(result);
                /*
            if (readXML.NodeType == XmlNodeType.Element)
            {
                switch(readXML.Name)
                {
                    case "info":
                        this.name = readXML.GetAttribute("face");
                        this.infoSize = long.Parse(readXML.GetAttribute("size"));
                        break;
                    case "common":
                        this.lineHeight = int.Parse(readXML.GetAttribute("lineHeight"));
                        this.commonBase = short.Parse(readXML.GetAttribute("base"));
                        this.textureWidth = int.Parse(readXML.GetAttribute("scaleW"));
                        this.textureHeight = int.Parse(readXML.GetAttribute("scaleH"));
                        break;
                    case "char":
                        let fontChar: FontCharacter = {
                            ASCIICode: int.Parse(readXML.GetAttribute("id")),
                            ID: this.Characters.Count,
                            X: short.Parse(readXML.GetAttribute("x")),
                            Y: short.Parse(readXML.GetAttribute("y")),
                            W: byte.Parse(readXML.GetAttribute("width")),
                            H: byte.Parse(readXML.GetAttribute("height")),
                            XOffset: sbyte.Parse(readXML.GetAttribute("xoffset")),
                            YOffset: short.Parse(readXML.GetAttribute("yoffset")),
                            XAdvance: byte.Parse(readXML.GetAttribute("xadvance")),
                        };
                        this.characters.push(fontChar);
                        break;
                }
            }
                */
            });
        });
    };
    DieselFont.prototype.readShortOrInt = function (br, large) {
        return large ? br.readInt() : br.readShort();
    };
    DieselFont.prototype.readInt = function (br, large) {
        return large ? br.readLong() : br.readInt();
    };
    DieselFont.prototype.writeLong = function (bw, i, large) {
        if (large)
            bw.writeLong(i);
        else
            bw.writeInt(i);
    };
    DieselFont.prototype.writeInt = function (bw, i, large) {
        if (large)
            bw.writeInt(i);
        else
            bw.writeShort(i);
    };
    DieselFont.prototype.ReadDieselFile = function (br, large) {
        //Normal: 92 bytes large: 168 bytes
        if (large === void 0) { large = false; }
        //Some values in raid seem to be complete 0 like u1.
        br.position = 0;
        var charCount = this.readInt(br, large); //Count
        this.readInt(br, large); //Count
        var fontRectPos = this.readInt(br, large);
        this.u1 = this.readInt(br, large); //Unknown(seems like in the case of system_font in raid it was turned into a zero and long lol)
        this.u2 = this.readInt(br, large); //Unknown
        this.readInt(br, large); //Count
        this.readInt(br, large); //Count
        var asciiCharsPos = this.readInt(br, large);
        this.readInt(br, large); //Unknown, same as pos 12 or 24 in raid.
        this.u3 = this.readInt(br, large); //Unknown
        this.u4 = this.readShortOrInt(br, large); //Unknown Short at end seems to be base
        this.commonBase = this.readShortOrInt(br, large);
        var kernings = this.readInt(br, large);
        this.readInt(br, large); //Unknown (the count of kernings again?)
        var kerningPos = this.readInt(br, large);
        var hasKerning = kerningPos != 0;
        this.readInt(br, large); //Unknown, same as pos 12
        this.u5 = this.readInt(br, large); //Unknown
        this.readInt(br, large); //Unknown, same as pos 12
        var endPos = this.readInt(br, large);
        var kernCount = (endPos - kerningPos) / 12;
        this.lineHeight = br.readInt(); //Line Height
        this.textureWidth = br.readInt();
        this.textureHeight = br.readInt();
        this.u7 = br.readInt(); //Sometimes the same as LineHeight
        this.infoSize = this.readInt(br, large); //base?
        if (br.position != fontRectPos) {
            console.warn("Position is not correct for Font Texture Rectangle by " + (br.position - fontRectPos));
            br.position = fontRectPos;
        }
        for (var i = 0; i < charCount; i++) {
            var fc = new FontCharacter();
            fc.read(br);
            this.characters.push(fc);
        }
        if ((charCount % 2) != 0)
            br.readShort();
        if (br.position != asciiCharsPos) {
            console.warn("Position is not correct for ASCII Characters by " + (br.position - asciiCharsPos));
            br.position = asciiCharsPos;
        }
        for (var i = 0; i < charCount; i++) {
            var ASCIICode = br.readInt();
            var ID = br.readInt();
            this.characters[ID].ID = ID;
            this.characters[ID].ASCIICode = ASCIICode;
        }
        if (hasKerning)
            for (var i = 0; i < kernCount; i++)
                this.kerning.push(new FontKerning(br));
        if (br.position != endPos) {
            console.warn("End Position isn't correct");
            br.position = endPos;
        }
        this.name = "";
        var ch;
        var cn;
        while (cn != 0) {
            ch = br.readChar(csharp_binary_stream_1.Encoding.Utf8);
            cn = ch.charCodeAt(0);
            console.log("character code..." + cn);
            if (cn != 0)
                this.name += ch;
        }
        br.readBytes((br.length - 4) - br.position);
        br.readInt(); //End Tag
    };
    DieselFont.prototype.WriteDieselData = function (bw, large) {
        if (large === void 0) { large = false; }
        var charCount = this.characters.length;
        this.writeLong(bw, charCount, large);
        this.writeLong(bw, charCount, large);
        var startPos = large ? 168 : 92;
        //font rect pos
        this.writeLong(bw, startPos, large);
        this.writeLong(bw, this.u1, large);
        this.writeLong(bw, this.u2, large);
        this.writeLong(bw, charCount, large);
        this.writeLong(bw, charCount, large);
        //asciiID pos
        this.writeLong(bw, startPos + (charCount * (large ? 12 : 10)) + ((charCount % 2) != 0 ? 2 : 0), large);
        this.writeLong(bw, this.u1, large);
        this.writeLong(bw, this.u3, large);
        this.writeInt(bw, this.u4, large);
        this.writeInt(bw, this.commonBase, large);
        var kernings = this.kerning.length;
        this.writeLong(bw, kernings, large);
        this.writeLong(bw, kernings, large);
        //Kerning pos
        this.writeLong(bw, (this.hasKerning() ? startPos + (charCount * 18) : 0) + ((charCount % 2) != 0 ? 2 : 0), large);
        this.writeLong(bw, this.u1, large);
        this.writeLong(bw, this.u5, large);
        this.writeLong(bw, this.u1, large);
        //end pos
        this.writeLong(bw, startPos + (charCount * 18) + (this.kerning.length * 12) + ((charCount % 2) != 0 ? 2 : 0), large);
        bw.writeInt(this.lineHeight);
        bw.writeInt(this.textureWidth);
        bw.writeInt(this.textureHeight);
        bw.writeInt(this.u7 != 0 ? this.lineHeight : this.u7);
        this.writeLong(bw, this.infoSize, large);
        this.characters.sort(function (x, y) {
            return x.ID - y.ID;
        });
        this.characters.forEach(function (fontChar) {
            fontChar.WriteMainSection(bw, large);
        });
        if ((charCount % 2) != 0)
            bw.writeShort(0);
        this.characters.sort(function (x, y) {
            return x.ASCIICode - y.ASCIICode;
        });
        this.characters.forEach(function (fontChar) {
            fontChar.WriteCodeIDSection(bw);
        });
        this.characters.forEach(function (fontChar) {
            fontChar.WriteCodeIDSection(bw);
        });
        this.kerning.forEach(function (fontKern) {
            fontKern.write(bw);
        });
        for (var i = 0; i < this.name.length; i++)
            bw.writeChar(this.name.charAt(i), csharp_binary_stream_1.Encoding.Utf8);
        bw.writeByte(0);
        while (bw.position % 4 != 0)
            bw.writeByte(0);
        bw.writeInt(endTag);
    };
    return DieselFont;
}());
exports.DieselFont = DieselFont;
